@@ -1,8 +1,10 @@
+# frozen_string_literal: true
+
 # All Plugin Dynamic Finders returning a Version are tested here.
 # When adding one to the spec/fixtures/db/dynamic_finder.yml, a few files have
 # to be edited/created
 #
-# - spec/fixtures/dynamic_finder/plugin_version/expected.yml with the expected result/s
+# - spec/fixtures/dynamic_finder/expected.yml with the expected result/s
 # - Then, depending on the finder class used: spec/fixtures/dynamic_finder/plugin_version/
 #
 # Furthermore, the fixtures files _passive_all.html are also used by plugins/themes
@@ -14,15 +16,9 @@
 
 expected_all = df_expected_all['plugins']
 
-WPScan::DB::DynamicFinders::Plugin.create_versions_finders
-
-describe 'Try to create the finders twice' do
-  it 'does not raise an error when the class already exists' do
-    expect { WPScan::DB::DynamicFinders::Plugin.create_versions_finders }.to_not raise_error
-  end
-end
-
 WPScan::DB::DynamicFinders::Plugin.versions_finders_configs.each do |slug, configs|
+  WPScan::DB::DynamicFinders::Plugin.create_versions_finders(slug)
+
   configs.each do |finder_class, config|
     finder_super_class = config['class'] || finder_class
 
@@ -30,9 +26,9 @@ WPScan::DB::DynamicFinders::Plugin.versions_finders_configs.each do |slug, confi
     # If someone find a fix for that, please share!
     describe df_tested_class_constant('PluginVersion', finder_class, slug), slow: true do
       subject(:finder) { described_class.new(plugin) }
-      let(:plugin)     { WPScan::Plugin.new(slug, target) }
+      let(:plugin)     { WPScan::Model::Plugin.new(slug, target) }
       let(:target)     { WPScan::Target.new('http://wp.lab/') }
-      let(:fixtures)   { File.join(DYNAMIC_FINDERS_FIXTURES, 'plugin_version') }
+      let(:fixtures)   { DYNAMIC_FINDERS_FIXTURES.join('plugin_version') }
 
       let(:expected) do
         if expected_all[slug][finder_class].is_a?(Hash)
@@ -42,13 +38,21 @@ WPScan::DB::DynamicFinders::Plugin.versions_finders_configs.each do |slug, confi
         end
       end
 
-      let(:stubbed_response) { { body: 'aa' } }
+      before { allow(target).to receive(:content_dir).and_return('wp-content') }
 
       describe '#passive', slow: true do
         before do
-          stub_request(:get, target.url).to_return(stubbed_response)
+          if defined?(stubbed_homepage_res)
+            stub_request(:get, target.url).to_return(stubbed_homepage_res)
+          else
+            stub_request(:get, target.url)
+          end
 
-          expect(target).to receive(:content_dir).at_least(1).and_return('wp-content')
+          if defined?(stubbed_404_res)
+            stub_request(:get, ERROR_404_URL_PATTERN).to_return(stubbed_404_res)
+          else
+            stub_request(:get, ERROR_404_URL_PATTERN)
+          end
         end
 
         if config['path']
@@ -60,27 +64,63 @@ WPScan::DB::DynamicFinders::Plugin.versions_finders_configs.each do |slug, confi
         else
           context 'when no PATH' do
             context 'when the version is detected' do
-              let(:stubbed_response) do
-                df_stubbed_response(
-                  File.join(fixtures, "#{finder_super_class.underscore}_passive_all.html"),
-                  finder_super_class
-                )
+              context 'from the homepage' do
+                let(:ie_url) { target.url }
+                let(:stubbed_homepage_res) do
+                  df_stubbed_response(
+                    fixtures.join("#{finder_super_class.underscore}_passive_all.html"),
+                    finder_super_class
+                  )
+                end
+
+                it 'returns the expected version/s' do
+                  found = Array(finder.passive)
+
+                  expect(found).to_not be_empty
+
+                  found.each_with_index do |version, index|
+                    expected_version = expected.at(index)
+                    expected_ie = expected_version['interesting_entries'].map do |ie|
+                      ie.gsub(target.url + ',', ie_url + ',')
+                    end
+
+                    expect(version).to be_a WPScan::Model::Version
+                    expect(version.number).to eql expected_version['number'].to_s
+                    expect(version.found_by).to eql expected_version['found_by']
+                    expect(version.interesting_entries).to match_array expected_ie
+
+                    expect(version.confidence).to eql expected_version['confidence'] if expected_version['confidence']
+                  end
+                end
               end
 
-              it 'returns the expected version/s from the homepage' do
-                found = [*finder.passive]
+              context 'from the 404' do
+                let(:ie_url) { target.error_404_url }
+                let(:stubbed_404_res) do
+                  df_stubbed_response(
+                    fixtures.join("#{finder_super_class.underscore}_passive_all.html"),
+                    finder_super_class
+                  )
+                end
 
-                expect(found).to_not be_empty
+                it 'returns the expected version/s' do
+                  found = Array(finder.passive)
 
-                found.each_with_index do |version, index|
-                  expected_version = expected.at(index)
+                  expect(found).to_not be_empty
 
-                  expect(version).to be_a WPScan::Version
-                  expect(version.number).to eql expected_version['number'].to_s
-                  expect(version.found_by).to eql expected_version['found_by']
-                  expect(version.interesting_entries).to match_array expected_version['interesting_entries']
+                  found.each_with_index do |version, index|
+                    expected_version = expected.at(index)
+                    expected_ie = expected_version['interesting_entries'].map do |ie|
+                      ie.gsub(target.url + ',', ie_url + ',')
+                    end
 
-                  expect(version.confidence).to eql expected_version['confidence'] if expected_version['confidence']
+                    expect(version).to be_a WPScan::Model::Version
+                    expect(version.number).to eql expected_version['number'].to_s
+                    expect(version.found_by).to eql expected_version['found_by']
+                    expect(version.interesting_entries).to match_array expected_ie
+
+                    expect(version.confidence).to eql expected_version['confidence'] if expected_version['confidence']
+                  end
                 end
               end
             end
@@ -95,29 +135,28 @@ WPScan::DB::DynamicFinders::Plugin.versions_finders_configs.each do |slug, confi
       end
 
       describe '#aggressive' do
-        let(:fixtures) { File.join(super(), slug, finder_class.underscore) }
+        let(:fixtures)         { super().join(slug, finder_class.underscore) }
+        let(:stubbed_response) { { body: 'aa' } }
 
         before do
-          expect(target).to receive(:content_dir).at_least(1).and_return('wp-content')
-
           stub_request(:get, plugin.url(config['path'])).to_return(stubbed_response) if config['path']
         end
 
         if config['path']
           context 'when the version is detected' do
             let(:stubbed_response) do
-              df_stubbed_response(File.join(fixtures, config['path']), finder_super_class)
+              df_stubbed_response(fixtures.join(config['path']), finder_super_class)
             end
 
             it 'returns the expected version' do
-              found = [*finder.aggressive]
+              found = Array(finder.aggressive)
 
               expect(found).to_not be_empty
 
               found.each_with_index do |version, index|
                 expected_version = expected.at(index)
 
-                expect(version).to be_a WPScan::Version
+                expect(version).to be_a WPScan::Model::Version
                 expect(version.number).to eql expected_version['number'].to_s
                 expect(version.found_by).to eql expected_version['found_by']
                 expect(version.interesting_entries).to match_array expected_version['interesting_entries']
